package com.hrsk.cloud.eg.infrastructure.exception;

import com.hrsk.cloud.eg.domain.common.constant.ResultCode;
import com.hrsk.cloud.eg.domain.common.response.DefaultResponseVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.bind.BindException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import javax.validation.ConstraintViolationException;


/**
 * 全局异常处理器
 */
@ControllerAdvice
@Slf4j
public class ExceptionAdvice {

    @ExceptionHandler(Exception.class)
    public ResponseEntity<DefaultResponseVo> exception(Exception exception) {
        DefaultResponseVo result = new DefaultResponseVo<>();
        ResponseEntity<DefaultResponseVo> springResponse = handleSpringException(exception);
        if (springResponse != null) {
            return springResponse;
        }

        if (exception instanceof IllegalArgumentException) {
            result.setCode(ResultCode.GLOBAL_REQUEST_FAIL.getCode());
            result.setMsg(exception.getMessage());
            return ResponseEntity.ok(result);
        }
        if (exception instanceof BindException) {
            result.setCode(ResultCode.GLOBAL_REQUEST_FAIL.getCode());
            result.setMsg(exception.getMessage());
            return ResponseEntity.ok(result);
        }
        if (exception instanceof ConstraintViolationException) {
            result.setCode(ResultCode.GLOBAL_REQUEST_FAIL.getCode());
            result.setMsg(exception.getMessage());
            return ResponseEntity.ok(result);
        }
        if (exception instanceof BindException) {
            BindException bizException = (BindException) exception;
            result.setCode(ResultCode.GLOBAL_REQUEST_FAIL.getCode());
            result.setMsg(ResultCode.GLOBAL_REQUEST_FAIL.getMsg());
            result.setData(null);
            return ResponseEntity.ok(result);
        }
        if (exception instanceof MethodArgumentNotValidException) {
            MethodArgumentNotValidException bindException = (MethodArgumentNotValidException) exception;
            result.setCode(400);
            result.setMsg(bindException.getBindingResult().getFieldError().getDefaultMessage());
            result.setData(null);
            return ResponseEntity.ok(result);
        }
        log.error("全局未知异常,异常信息：{}", exception.getMessage(), exception);
        result = DefaultResponseVo.fail(ResultCode.GLOBAL_REQUEST_FAIL.getMsg());
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(result);
    }

    private ResponseEntity<DefaultResponseVo> handleSpringException(Exception exception) {
        if (exception instanceof ServletRequestBindingException) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(DefaultResponseVo.fail(exception.getMessage()));
        }
        if (exception instanceof HttpRequestMethodNotSupportedException) {
            return ResponseEntity.status(HttpStatus.METHOD_NOT_ALLOWED).body(DefaultResponseVo.fail(exception.getMessage()));
        }
        if (exception instanceof HttpMessageNotReadableException) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(DefaultResponseVo.fail(exception.getMessage()));
        }
        return null;
    }
}